/*
* Copyright 2011 the original author or authors.
*
* Licensed under the Apache License, Version 2.0 (the "License"); you may not
* use this file except in compliance with the License. You may obtain a copy of
* the License at
*
* http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
* WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
* License for the specific language governing permissions and limitations under
* the License.
*/

package com.github.javarch.jsf.context;

import java.util.List;

import javax.faces.application.FacesMessage;
import javax.faces.application.FacesMessage.Severity;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.validation.BindingResult;
import org.springframework.validation.FieldError;

import com.github.javarch.jsf.CurrentLocale;
import com.github.javarch.support.MessageSourceAware;
import com.google.common.base.Preconditions;

/**
 * Publish I18N messages in FacesContext.
 * 
 * @author Lucas 
 */
@Component
public class FacesMessageContext {
	 
	@Autowired
	public FacesContextUtils facesContext;
	 
	@Autowired
	private CurrentLocale locale;
	
	@Autowired
	private MessageSourceAware messageSourceAware;
		

	/**
	 * Adiciona a mensagem da variável <i>msg</i> ao FacesContext. A mensagem adicionada terá
	 * o severity do tipo FacesMessage.SEVERITY_INFO e não será associada à nenhum componente.
	 * 
	 * @param msg Mensagem que será adicionada ao FacesContext
	 */
	public void addInfo(String code, Object ... params) {
		add(null,code, FacesMessage.SEVERITY_INFO , params);
	}


	public void addInfo(String idComponent, String code, Object ...params ){
		add(idComponent,code, FacesMessage.SEVERITY_INFO , params);
	}

	/**
	 * Adiciona a mensagem da variável <i>msg</i> ao FacesContext. A mensagem adicionada terá
	 * o severity do tipo FacesMessage.SEVERITY_ERROR e não será associada à nenhum componente.
	 * 
	 * @param msg Mensagem que será adicionada ao FacesContext
	 */
	public void addError(String code, Object ... params) {
		add(null, code,  FacesMessage.SEVERITY_ERROR, params);
	}
		
	public void addError(String idComponent, String code, Object ...params ){
		add(idComponent, code,  FacesMessage.SEVERITY_ERROR, params);
	}
	
	private void add(String componentId, String code, Severity tipo, Object ... params) {		
		if (  code != null && !code.isEmpty() ){		
			String[] paramsAsString = parseArrayObjectToArrayString( params );
			String mensagemFormatada = getMessageI18n(code, paramsAsString );		
			FacesMessage facesMessage = new FacesMessage( tipo , mensagemFormatada, "");
			facesContext.getFacesContext().addMessage( componentId , facesMessage );
		}
	}
	
	private String[] parseArrayObjectToArrayString(Object[] params) {
		int paramsLength = params.length;
		String[] paramsAsString =  new String[ paramsLength ];
		for( int i = 0 ; i < paramsLength ; i++ ){
			paramsAsString[i] =  params[i].toString();
		}		
		return paramsAsString;
	}

	protected String getMessageI18n(String code, String ... args ){
		return messageSourceAware.getMessage(code, args, locale.getLocale() );
	}
	
	public List<FacesMessage> getMessages() {
		return facesContext.getFacesContext().getMessageList();
	}
 
	public void clear() {
		facesContext.getFacesContext().getMessageList().clear();		
	}

	public void addError(BindingResult erros){
		Preconditions.checkArgument(erros != null, "BindingResult não pode ser nulo.");
						
		for (FieldError fieldError : erros.getFieldErrors()) {						
			addError( fieldError.getDefaultMessage() );				             				
		}			
	}
	
	
	public void setFacesContext(FacesContextUtils facesContext) {
		Preconditions.checkArgument(facesContext != null, "FacesContext can not be null.");
		this.facesContext = facesContext;
	}
	
	public void setMessageSourceAware(MessageSourceAware messageSourceAware) {
		Preconditions.checkArgument(messageSourceAware != null, "MessageSourceAware can not be null.");
		this.messageSourceAware = messageSourceAware;
	}
	
	public void setCurrentLocale(CurrentLocale locale) {
		Preconditions.checkArgument(locale != null, "CurrentLocale can not be null.");
		this.locale = locale;
	}
}
