package com.github.javarch.jsf.profiling;

import java.io.File;
import java.io.IOException;
import java.util.concurrent.TimeUnit;

import javax.servlet.FilterChain;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.springframework.core.env.Environment;
import org.springframework.web.context.WebApplicationContext;
import org.springframework.web.context.support.WebApplicationContextUtils;

import com.github.javarch.jsf.context.OncePerRequestFacesFilter;
import com.github.javarch.support.log.Logger;
import com.github.javarch.support.log.LoggerFactory;
import com.github.javarch.support.spring.Profiles;
import com.yammer.metrics.core.Meter;
import com.yammer.metrics.core.MetricsRegistry;
import com.yammer.metrics.core.Timer;
import com.yammer.metrics.core.TimerContext;
import com.yammer.metrics.reporting.ConsoleReporter;
import com.yammer.metrics.reporting.CsvReporter;

public class RequestAccessFilter extends OncePerRequestFacesFilter  {
	
	private MetricsRegistry registry;
	private Meter requests;
	private Timer timer;
	private boolean profiling;
	private final Logger logger = LoggerFactory.getLogger(RequestAccessFilter.class);
	
	@Override
	public void initFilterBean() throws ServletException {
		
		logger.debug("Initializing {}", getClass().getSimpleName() );
		
		WebApplicationContext springContext = 
			        WebApplicationContextUtils.getWebApplicationContext(getServletContext());
		
		Environment environment = springContext.getEnvironment();
		
		if( environment.acceptsProfiles( Profiles.PROFILING ) && springContext.containsBean("metricsRegistry") ){						
			profiling = true;
			registry = springContext.getBean("metricsRegistry",MetricsRegistry.class);
			requests = registry.newMeter(RequestAccessFilter.class, "metrics-amount-requests", "requests", TimeUnit.SECONDS );
			timer =  registry.newTimer( RequestAccessFilter.class, "request-time", "requests" );								
			
			String reportMode = getFilterConfig().getInitParameter("REPORT_MODE");
			
			if( reportMode.equalsIgnoreCase("Console") ){
				ConsoleReporter.enable(registry, 1, TimeUnit.MINUTES);
			} else if ( reportMode.equalsIgnoreCase("Csv") ){
				String reportPath = getFilterConfig().getInitParameter("REPORT_PATH");
				File outputDir = new File( reportPath );
				CsvReporter.enable(registry, outputDir, 1, TimeUnit.MINUTES );
			} else {
				logger.warn("Metrics Reporter not configured. You need define one of these alternatives: Console, Cvs");
			}
			
		} else {
			logger.warn("MetricsRegistry not found in ApplicationContext. {} will not collect metrics.", getClass().getName() );
		}				 
	}
 
	@Override
	protected void doFacesFilterInternal(HttpServletRequest request,
			HttpServletResponse response, FilterChain filterChain)
			throws ServletException, IOException {
		
		TimerContext context = null;
	
		try {
			if ( profiling ){
				context = timer.time();
				requests.mark();			
			}	
			// process
			filterChain.doFilter(request, response);
		}finally {
			if( profiling ){
				context.stop();
			}
		}
	}
}
